<?php

declare(strict_types=1);

namespace apexl\Io\modules\system\collections;

use apexl\Io\interfaces\IoModuleInterface;
use Arrayy\Arrayy;
use Arrayy\Collection\AbstractCollection;

/**
 * @extends Arrayy<array-key, IoModuleInterface>
 */
final class ModuleCollection extends AbstractCollection
{

    public function getType(): string
    {
        return IoModuleInterface::class;
    }
}