<?php

declare(strict_types=1);

namespace apexl\Io\exceptions;

use JsonException;
use Throwable;

class JsonRecursionException extends JsonException
{
    public function __construct(public array $data, string $message = "", int $code = 0, ?Throwable $previous = null)
    {
        parent::__construct($message, $code, $previous);
    }
}